@@ -70,15 +70,15 @@ $DocsParser::commentEnd = "   */";
 sub read_defs($$$)
 {
   my ($path, $filename, $filename_override) = @_;
-  
+
   my $objParser = new XML::Parser(ErrorContext => 0);
   $objParser->setHandlers(Start => \&parse_on_start, End => \&parse_on_end, Char => \&parse_on_cdata);
-  
+
   # C documentation:
   $DocsParser::CurrentFile = "$path/$filename";
   if ( ! -r $DocsParser::CurrentFile)
   {
-     print "DocsParser.pm: Warning: Can't read file \"" . $DocsParser::CurrentFile . "\".\n";
+     print STDERR "DocsParser.pm: Warning: Can't read file \"" . $DocsParser::CurrentFile . "\".\n";
      return;
   }
   # Parse
@@ -86,23 +86,22 @@ sub read_defs($$$)
   if( $@ )
   {
     $@ =~ s/at \/.*?$//s;
-    print "\nError in \"" . $DocsParser::CurrentFile . "\":$@\n";
+    print STDERR "\nError in \"" . $DocsParser::CurrentFile . "\":$@\n";
     return;
   }
 
   # C++ overide documentation:
-  $DocsParser::CurrentFile = "$path/$filename_override";
-  if ( ! -r $DocsParser::CurrentFile)
-  {
-     print "DocsParser.pm: Warning: Can't read file \"" . $DocsParser::CurrentFile . "\".\n";
-     return;
-  }
+  $DocsParser::CurrentFile = $path . '/' . $filename_override;
+
+  # It is not an error if the documentation override file does not exist.
+  return unless (-r $DocsParser::CurrentFile);
+
   # Parse
   eval { $objParser->parsefile($DocsParser::CurrentFile) };
   if( $@ )
   {
     $@ =~ s/at \/.*?$//s;
-    print "\nError in \"" . $DocsParser::CurrentFile . "\":$@";
+    print STDERR "\nError in \"" . $DocsParser::CurrentFile . "\":$@";
     return;
   }
 }
@@ -119,7 +118,7 @@ sub parse_on_start($$%)
     {
       $objParser->xpcroak("\nClose a function tag before you open another one.");
     }
-    
+
     my $functionName = $attr{name};
 
     #Reuse existing Function, if it exists:
@@ -250,7 +249,6 @@ sub lookup_documentation($$)
     print "DocsParser.pm: Warning: No C docs for function: \"$functionName\"\n";
   }
 
-
   DocsParser::convert_docs_to_cpp($objFunction, \$text);
 
   #Add note about deprecation if we have specified that in our _WRAP_METHOD() call:
@@ -261,11 +259,20 @@ sub lookup_documentation($$)
 
   DocsParser::append_parameter_docs($objFunction, \$text);
   DocsParser::append_return_docs($objFunction, \$text);
-
+  DocsParser::add_m4_quotes(\$text);
 
   # Escape the space after "i.e." or "e.g." in the brief description.
   $text =~ s/^([^.]*\b(?:i\.e\.|e\.g\.))\s/$1\\ /;
 
+  # Remove C example code.
+  my $example_removals =
+    ($text =~ s"<informalexample>.*?</informalexample>""sg);
+
+  $example_removals += ($text =~ s"\|\[.*?]\|""sg);
+
+  print STDERR "gmmproc: $functionName(): Example code discarded.\n"
+    if ($example_removals);
+
   # Convert to Doxygen-style comment.
   $text =~ s/\n/\n${DocsParser::commentMiddleStart}/g;
   $text =  $DocsParser::commentStart . $text;
@@ -274,6 +281,20 @@ sub lookup_documentation($$)
   return $text;
 }
 
+sub add_m4_quotes($)
+{
+  my ($text) = @_;
+
+  # __BT__ and __FT__ are M4 macros defined in the base.m4 file that produce
+  # a "`" and a "'" resp. without M4 errors.
+  my %m4_quotes = (
+    "`" => "'__BT__`",
+    "'" => "'__FT__`",
+  );
+
+  $$text =~ s/([`'])/$m4_quotes{$1}/g;
+  $$text = "`" . $$text . "'";
+}
 
 sub append_parameter_docs($$)
 {
@@ -294,7 +315,7 @@ sub append_parameter_docs($$)
     if ($param ne "error" ) #We wrap GErrors as exceptions, so ignore these.
     {
       my $desc = $$param_descriptions->{$param};
-    
+
       $param =~ s/([a-zA-Z0-9]*(_[a-zA-Z0-9]+)*)_?/$1/g;
       DocsParser::convert_docs_to_cpp($obj_function, \$desc);
       if(length($desc) > 0)
@@ -347,9 +368,10 @@ sub convert_tags_to_doxygen($)
   for($$text)
   {
     # Replace format tags.
-    s"&lt;(/?)emphasis&gt;"<$1em>"g;
-    s"&lt;(/?)literal&gt;"<$1tt>"g;
-    s"&lt;(/?)function&gt;"<$1tt>"g;
+    s"<(/?)emphasis>"<$1em>"g;
+    s"<(/?)literal>"<$1tt>"g;
+    s"<(/?)constant>"<$1tt>"g;
+    s"<(/?)function>"<$1tt>"g;
 
     # Some argument names are suffixed by "_" -- strip this.
     # gtk-doc uses @thearg, but doxygen uses @a thearg.
@@ -360,21 +382,20 @@ sub convert_tags_to_doxygen($)
     s" \@a (throws|param)" \@$1"g;
     s"^Note ?\d?: "\@note "mg;
 
-    s"&lt;/?programlisting&gt;""g;
-    s"&lt;informalexample&gt;"\@code"g;
-    s"&lt;/informalexample&gt;"\@endcode"g;
-    s"&lt;!&gt;""g;
+    s"</?programlisting>""g;
+
+    s"<!>""g;
 
     # Remove all link tags.
-    s"&lt;/?u?link[^&]*&gt;""g;
+    s"</?u?link[^&]*?>""g;
 
     # Remove all para tags (from tmpl sgml files).
-    s"&lt;/?para&gt;""g;
+    s"</?para>""g;
 
     # Use our Doxygen @newin alias:
     s/\bSince:\s*(\d+)\.(\d+)\b/\@newin{$1,$2}/g;
 
-    s"\b-&gt;\b"->"g;
+    s"\b->\b"->"g;
 
     # Doxygen is too dumb to handle &mdash;
     s"&mdash;" \@htmlonly&mdash;\@endhtmlonly "g;
@@ -431,7 +452,7 @@ sub substitute_function($$)
     }
     else
     {
-      print "Documentation: Transformed C name $name into ";  
+      print "Documentation: Transformed C name $name into ";
       non_object_method_name($doc_func, \$name);
       print "C++ name $name\n";
     }
@@ -448,7 +469,7 @@ sub substitute_function($$)
 sub non_object_method_name($$)
 {
   my ($doc_func, $name) = @_;
-  if ($$name =~ "^gtk_")
+  if ($$name =~ m/^gtk_/)
   {
     my %gtk_objects = ("gtk_accel_map" => "AccelMap",
                        "gtk_clipboard" => "Clipboard",
@@ -465,16 +486,16 @@ sub non_object_method_name($$)
                        "gtk_recent_filter" => "RecentFilter");
     foreach my $key (keys(%gtk_objects))
     {
-      if ($$name =~ "^$key")
+      if ($$name =~ m/^\Q$key\E/)
       {
         DocsParser::build_method_name($doc_func, "Gtk", $gtk_objects{$key}, $name);
         return;
       }
     }
   }
-  
+
   print STDERR "Documentation: Class/Namespace for $$name not found\n";
-}   
+}
 
 sub lookup_object_of_method($$)
 {
@@ -491,8 +512,8 @@ sub lookup_object_of_method($$)
     }
     else
     {
-      print "DocsParser.pm:lookup_object_of_method(): Warning: GtkDefs::lookup_object() failed for function name=" . $name . "\n";
-      print "  This may be a missing define-object in a *.defs file."
+      print "DocsParser.pm:lookup_object_of_method(): Warning: GtkDefs::lookup_object() failed for object name=" . $object . ", function name=" . $name . "\n";
+      print "  This may be a missing define-object in a *.defs file.\n"
     }
   }
 
@@ -503,7 +524,7 @@ sub lookup_object_of_method($$)
   foreach(@parts) { $_ = (length > 2) ? ucfirst : uc; }
 
   # Do a bit of try'n'error.
-  while(scalar(@parts) > 1)
+  while($#parts >= 1)
   {
     my $try = join("", @parts);
 
@@ -512,7 +533,7 @@ sub lookup_object_of_method($$)
 
     pop(@parts);
   }
-  
+
   return undef;
 }
 
@@ -526,10 +547,10 @@ sub build_method_name($$$$)
   $prefix =~ s/([a-z])([A-Z])/$1_$2/g;
   $prefix = lc($prefix) . '_';
 
-  if($$name =~ /^$prefix/)
+  if($$name =~ m/^\Q$prefix\E/)
   {
     my $scope = "";
-    $scope = "${module}::${class}::" unless($doc_func =~ /^$prefix/);
+    $scope = "${module}::${class}::" unless($doc_func =~ m/^\Q$prefix\E/);
 
     substr($$name, 0, length($prefix)) = $scope;
   }
